// HSV
// Paulo Ricardo Reinehr
// PS 2.0
// Adjust Hue, Saturation and Value.


//#PARAM float h 0 : Hue : Hue shift in degrees (180 = opposite hues). Accept negative values.
float h;

//#PARAM percent s 1 : Saturation : Color saturation percentage. Values bigger than 100% oversaturate colors. 
float s;

//#PARAM percent v 1 : Value : Value percentage. Adjust the "lightness".
float v;


// Foreground texture
texture ForegroundTexture;

// Foreground sampler
sampler2D foreground = sampler_state {
    Texture = (ForegroundTexture);
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = Point;
};


// Effect function
float4 EffectProcess( float2 Tex : TEXCOORD0 ) : COLOR0
{
	float4 color = tex2D(foreground, Tex.xy);

// store input color	
	float3 input = { color.r , color.g , color.b };

// convert hue in degrees and set saturation and value minimum to 0
	float h = fmod( h , 360 );
	float s = max( 0 , s );
	float v = max( 0 , v );
	
// calculate VSU and VSW
	static const float PI = 3.14159265;
	
	float VSU = v * s * cos(h * PI / 180.0);
	float VSW = v * s * sin(h * PI / 180.0);
	
// create transformation matrix
	float3x3 hsv =
	{
		(0.299*v)+(0.701*VSU)+(0.168*VSW),
		(0.587*v)-(0.587*VSU)+(0.330*VSW),
		(0.114*v)-(0.114*VSU)-(0.497*VSW),
		(0.299*v)-(0.299*VSU)-(0.328*VSW),
		(0.587*v)+(0.413*VSU)+(0.035*VSW),
		(0.114*v)-(0.114*VSU)+(0.292*VSW),
		(0.299*v)-(0.3*VSU)+(1.25*VSW),
		(0.587*v)-(0.588*VSU)-(1.05*VSW),
		(0.114*v)+(0.886*VSU)-(0.203*VSW)
	};
	
// multiplies the transformation matrix with the input color
	color.rgb = mul( hsv, input );
	
	return color;
}

// ConstructEffect
technique ConstructEffect
{
    pass p0
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 EffectProcess();    
    }
}
